// maxADC library
// Version date: Dec 17, 2021
// interface with MAX ADC chips for TC4 project
// based on Jim Gallit cADC library


// *** BSD License ***
// ------------------------------------------------------------------------------------------
// Copyright (c) 2010, MLG Properties, LLC
// All rights reserved.
//
// Contributor:  Jim Gallt, Renato Aranghelovici
//
// Redistribution and use in source and binary forms, with or without modification, are 
// permitted provided that the following conditions are met:
//
//   Redistributions of source code must retain the above copyright notice, this list of 
//   conditions and the following disclaimer.
//
//   Redistributions in binary form must reproduce the above copyright notice, this list 
//   of conditions and the following disclaimer in the documentation and/or other materials 
//   provided with the distribution.
//
//   Neither the name of the MLG Properties, LLC nor the names of its contributors may be 
//   used to endorse or promote products derived from this software without specific prior 
//   written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
// OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
// THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// ------------------------------------------------------------------------------------------

// Acknowledgement is given to Bill Welch for his development of the prototype hardware and software 
// upon which much of this library is based.

// Revision history:
//  Original version date: 17-Dec-2021
// 20220317: added MAX31856 support
// 20220320: added MAX31865 Pt support

#include "maxADC.h"

#if defined(ARDUINO) && ARDUINO >= 100
#define _READ read
#define _WRITE write
#else
#define _READ receive
#define _WRITE send
#endif


#if defined ESP8266
#if defined using_MAX6675
#include <max6675.h>
MAX6675 max1(max_CLCK, max_CSEL, max_MISO);
#if MAX_CHANNELS > 1 && defined max_CS2
MAX6675 max2(max_CLCK, max_CS2, max_MISO);
#endif  //  2nd MAX6675
#endif  //  MAX6675
//#endif  //  ESP8266

#elif defined ESP32
#if defined using_MAX6675
#include <max6675.h>
MAX6675 max1(max_CLCK, max_CSEL, max_MISO);
#if MAX_CHANNELS > 1 && defined max_CS2
MAX6675 max2(max_CLCK, max_CS2, max_MISO);
#endif  //  2nd MAX6675 connected
// #endif  //  MAX6675

#elif defined using_MAX31856
#include <Adafruit_MAX31856.h>
Adafruit_MAX31856 max1 = Adafruit_MAX31856(max_CSEL, max_MOSI, max_MISO, max_CLCK);

#elif defined using_MAX31865
#include <Adafruit_MAX31865.h>
Adafruit_MAX31865 max1 = Adafruit_MAX31865(max_CSEL, max_MOSI, max_MISO, max_CLCK);
// The value of the Rref resistor. Use 430.0 for PT100 and 4300.0 for PT1000
#define RREF      430.0
// The 'nominal' 0-degrees-C resistance of the sensor
// 100.0 for PT100, 1000.0 for PT1000
#define RNOMINAL  100.0

#endif  //  MAX chips enum

#endif  //  ESP32


// ----------------------------------------------------------
maxADC::maxADC( uint8_t addr ) {

	//a_adc = addr; // address of ADC chip
  cal_gain = CAL_GAIN;
  cal_offset = CAL_OFFSET;

  setCfg(ADC_BITS_16, ADC_GAIN_1, ADC_CONV_1SHOT); // 16 bit, 1X, 1 shot

}

// --------------------------------------------------setCfg
void maxADC::setCfg( uint8_t res, uint8_t gain, uint8_t conv  ){
  cfg = res | gain | conv;

#if defined using_MAX6675
  convTime = _CONV_TIME_6675;

#elif defined using_MAX31855
  convTime = _CONV_TIME_31855;

#elif defined using_MAX31856
  convTime = _CONV_TIME_31856;

  if (!max1.begin()) {
	  Serial.println("# Could not initialize thermocouple.");
	  // while (1) delay(10);
  }

//#if defined usingK // tbd: switch for various TC types
  max1.setThermocoupleType(MAX31856_TCTYPE_K);

#ifdef ADC_CONV_1SHOT
  max1.setConversionMode(MAX31856_ONESHOT_NOWAIT);
#endif // ADC_CONV_1SHOT

#elif defined using_MAX31865
  convTime = _CONV_TIME_31865;
  max1.begin(MAX31865_3WIRE);  // set to 2WIRE or 4WIRE as necessary

#endif // using_MAX.....
}

// -------------------------------------------------- getConvTime
uint16_t maxADC::getConvTime() {
  return convTime;
}

// --------------------------------------------------setCal
void maxADC::setCal( float gain, int8_t offs ) {
  cal_gain = gain - 1.0;  // to reduce loss of significance
  cal_offset = offs;
};

// -----------------------------------------------------------
int32_t maxADC::readuV() {
  int32_t v;


  if (maxIdx == 0)
#if defined using_MAX6675
	  v = max1.readCelsius();
#elif defined using_MAX31856
	  v = max1.readThermocoupleTemperature();
	  uint8_t fault = max1.readFault();
	  if (fault) {
			// TBD fault management
			// max1.clearFault();
	  }
#elif defined using_MAX31865
	  v = max1.temperature(RNOMINAL, RREF);
	  uint8_t fault = max1.readFault();
	  if (fault) {
		  // TBD fault management
		  max1.clearFault();
	  }
#endif

#if MAX_CHANNELS > 1
  else if (maxIdx == 1)
#if defined max_CS2
#if defined using_MAX6675
	  v = max2.readCelsius();
#elif defined using_MAX31856
	  v = max2.readThermocoupleTemperature();
#elif defined using_MAX31865
	  v = max2.temperature(RNOMINAL, RREF);
#endif
#else
	v = 200; // fake virtual ET when no second max
#endif // second max channel

#endif // more than 1 max channel/device

	// Serial.print("MAXidx " + (String)maxIdx); Serial.println(" V " + (String)v);

  v *= 5000;  // convert to uV, assuming a linear 5mV/ K degree scale, as AD849x.  
			  
	// bit shift count for ADC gain
  //uint8_t gn = stat & ADC_GAIN_MASK;
  //// shift based on ADC resolution plus ADC gain
  //v >>= (nLSB + gn); // v = raw reading, uV

  // calculate effect of external calibration gain; minimize loss of significance
  int32_t deltaV = round( (float)v * cal_gain );
  // Serial.print("uV " + (String)v); Serial.println(" deltaV " + (String)deltaV);
  return v + deltaV;  // returns corrected, unfiltered value of uV
};

// -------------------------------------
void maxADC::nextConversion( uint8_t chan ) {

	// Serial.print("MAXidx chan "); Serial.println(chan);
	maxIdx = chan;
	// maybe a conversion start millis stored somewhere would be useful...

#if defined using_MAX31856 && defined ADC_CONV_1SHOT
	max1.triggerOneShot();
#endif
};


/*
#pragma region ambSensor
// ----------------------------------------------------------- ambSensor
// not using ambient sensor, just placeholder methods

ambSensor::ambSensor( uint8_t addr ) {
 //a_amb = addr; // I2C address
 // filter.init( 0 );
 setCfg(12); 
}

// -----------------------------------------
// sets up the configuration byte
void ambSensor::setCfg( uint8_t res = 12) {

  cfg = res | convMode;

#if defined using_MAX6675
  convTime = _CONV_TIME_6675;
#elif defined using_MAX31855
  convTime = _CONV_TIME_31855;
#elif defined using_MAX31856
  convTime = _CONV_TIME_31856;
#elif defined using_MAX31865
  convTime = _CONV_TIME_31865;
#endif

}

// -----------------------------------------
// returns minimum time required for a conversion
uint16_t ambSensor::getConvTime() {
	return convTime;

}

// -----------------------------------------
void ambSensor::ambShutdown() {
	
}


// ------------------------------------------------
void ambSensor::init( int fpercent, uint8_t cmode ) {
  // filter.init( fpercent );
  convMode = cmode;
  if( cmode == AMB_CONV_1SHOT )
    ambShutdown();
}

// -----------------------------------------
void ambSensor::nextConversion() {
	// maybe a conversion start millis stored somewhere would be useful...
}

// -----------------------------------------
int32_t ambSensor::readSensor() {

	filtered = 25;
	ambC = filtered;
	ambF = 1.8 * ambC + 32.0 ;

  return filtered;
};

// -----------------------------------
float ambSensor::getAmbC() { return ambC; }
float ambSensor::getAmbF() { return ambF; }

// ------------------------------------
float ambSensor::getOffset() {
 return temp_offset;
};

// ---------------------------------------
void ambSensor::setOffset( float tempC ) {
 temp_offset = tempC;
};

#pragma endregion ambSensor
*/

#undef _READ
#undef _WRITE

