// maxADC library

// interface with MAX ADC chips for TC4 project
// based on Jim Gallit cADC library

// *** BSD License ***
// ------------------------------------------------------------------------------------------
// Copyright (c) 2021-2022
// All rights reserved.
//
// Contributor:  Renato Aranghelovici
//
// Redistribution and use in source and binary forms, with or without modification, are 
// permitted provided that the following conditions are met:
//
//   Redistributions of source code must retain the above copyright notice, this list of 
//   conditions and the following disclaimer.
//
//   Redistributions in binary form must reproduce the above copyright notice, this list 
//   of conditions and the following disclaimer in the documentation and/or other materials 
//   provided with the distribution.
//
//   Neither the name of the MLG Properties, LLC nor the names of its contributors may be 
//   used to endorse or promote products derived from this software without specific prior 
//   written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
// OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
// THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// ------------------------------------------------------------------------------------------

// Acknowledgement is given to Bill Welch for his development of the prototype hardware and software 
// upon which much of this library is based.
 
// Revision history:
//  Original version date: 17-Dec-2021
// 20220317: added MAX31856 support
// 20220320: added MAX31865 Pt support
// 20230117: added ESP32 VSPI pinout

#ifndef _maxADC_H
#define _maxADC_H

#include <Wire.h>

#if defined(ARDUINO) && ARDUINO >= 100
#include <Arduino.h>
#else
#include <WProgram.h>
#endif


// #define using_MAX6675
// #define using_MAX31855
// #define using_MAX31856 // TC
// #define using_MAX31865 // Pt

#if defined CUST_FBMUSE
#define using_MAX31856 // TC
#elif defined CUST_PROGEN
#if (PROGEN_TYPE == 1)
#define using_MAX31865 // Pt
#endif
#endif

#define MAX_CHANNELS 2 // maximum number of channels to sample

#if defined ESP8266
#if defined using_MAX6675
#define max_MISO  12  // ESP8266 SPI pins
#define max_CLCK  14  
#define max_CSEL  15  
#if MAX_CHANNELS > 1
#define max_CS2  13  // SPI CS of 2nd MAX chip
#endif  //  2nd MAX6675
#endif  //  MAX6675

#elif defined ESP32
// ESP32 HSPI pins
//#define max_MISO  12  
//#define max_CLCK  14  
//#define max_CSEL  15   
//#if defined using_MAX31856 || defined using_MAX31865
//#define max_MOSI  13  // MOSI used for MAX31856/65 only
// ESP32 VSPI pins 
#define max_MISO  19  
#define max_CLCK  18  
#define max_CSEL  5   
#if defined using_MAX31856 || defined using_MAX31865
#define max_MOSI  23  // MOSI used for MAX31856/65 only
#endif  //  MAX31856/65
#if MAX_CHANNELS > 1
// #define max_CS2  5  // SPI CS of 2nd MAX chip
#endif  //  2nd MAX6675

#else // Uno
#if defined using_MAX6675
#define max_MISO  ...  // Uno SPI pins
#define max_CLCK  ...  
#define max_CSEL  ...  
#if MAX_CHANNELS > 1
#define max_CS2  ..  // SPI CS of 2nd MAX chip
#endif  //  2nd MAX6675
#endif  //  MAX6675
#endif  //  Uno


// -------------- ADC configuration
#define ADC_RDY 7  // ready bit
#define ADC_C1  6  // channel selection bit 1
#define ADC_C0  5  // channel selection bit 0
#define ADC_CMB 4  // conversion mode bit (1 = continuous 0 = one shot)
#define ADC_SR1 3  // sample rate selection bit 1 (11 = 18 bit)
#define ADC_SR0 2  // sample rate selection bit 0
#define ADC_G1  1  // gain select bit 1
#define ADC_G0  0  // gain select bit 0

#define ADC_GAIN_MASK ( _BV(ADC_G1) | _BV(ADC_G0) )
#define ADC_RES_MASK  ( _BV(ADC_SR1) | _BV(ADC_SR0) )

// some 3424 config register values
#define ADC_BITS_18 ( _BV(ADC_SR1) | _BV(ADC_SR0) )
#define ADC_BITS_16 ( _BV(ADC_SR1) )
#define ADC_BITS_14 ( _BV(ADC_SR0) )
#define ADC_BITS_12 0
#define ADC_GAIN_8 ( _BV(ADC_G1) | _BV(ADC_G0) )
#define ADC_GAIN_4 ( _BV(ADC_G1) )
#define ADC_GAIN_2 ( _BV(ADC_G0) )
#define ADC_GAIN_1 0
#define ADC_CHAN_4 ( _BV(ADC_C1) | _BV(ADC_C0) )
#define ADC_CHAN_3 ( _BV(ADC_C1) )
#define ADC_CHAN_2 ( _BV(ADC_C0) )
#define ADC_CHAN_1 0

#define ADC_CONV_CONT  ( _BV(ADC_CMB) )
#define ADC_CONV_1SHOT ( _BV(ADC_RDY) )

// calibration of ADC
#define CAL_OFFSET  ( 0 )  // microvolts
#define CAL_GAIN 1.00

// ADC I2C address
// #define A_ADC 0x68

// #define BITS_TO_uV 15.625  // LSB = 15.625 uV, 18 bit mode
// #define MAX_CHAN 1 // maximum number of channels to sample

#define _CONV_TIME_6675 250 // millis
#define _CONV_TIME_31855 250 // millis
#define _CONV_TIME_31856 250 // millis
#define _CONV_TIME_31865 21 // millis

// --------------------------------------------------------------
class maxADC {
 public:
  maxADC( uint8_t addr ); // constructor
  int32_t readuV(); // retrieves sample and converts to uV
  void nextConversion( uint8_t chan );  // requests the next conversion
  void setCal( float gaincal, int8_t offs ); // sets calibration gain/offset for 50000 uV and 0 uV
  void setCfg( uint8_t resolution, uint8_t gain, uint8_t conversion = ADC_CONV_1SHOT );
  uint16_t getConvTime(); // returns time required for a conversion
 protected:
  uint8_t cfg; // resolution, gain, and conversion mode settings
  uint8_t a_adc;
  float cal_gain; // = calibration factor minus 1.0000
  int8_t cal_offset;
  uint16_t convTime;  // milliseconds
//  float xLSB;  // microvolt value of LSB
  // LSB uV = 1,000 * 2^12 / 2^n, where n = ADC resolution bits
  uint8_t nLSB; // shift count = resolution bits minus 12
  uint8_t maxIdx;
};

// #include <ambSensor.h>

#endif
